# python-jinjatools
#
# various tools for Jinja2
# including new filters and tests based on python-moretools,
# a JinjaLoader class for Django
# and a simple JinjaBuilder class for SCons
#
# Copyright (C) 2011-2015 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# python-jinjatools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-jinjatools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with python-jinjatools.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['filters']

from moretools import *


def do_dictremove(dict_, *keys):
    for key in keys:
        try:
            del dict_[key]
        except KeyError:
            pass
    return dict_


def do_dictupdate(dict_, items=None, **kwitems):
    dict_.update(items or (), **kwitems)
    return dict_


def do_prefix(string, prefix):
    return prefix + string


def do_suffix(string, suffix):
    return string + suffix


filters = {
  'mapattr': mapattr,
  'mapmapattr': mapmapattr,

  'mapattrs': mapattrs,
  'mapmapattrs': mapmapattrs,

  'mapitem': mapitem,
  'mapmapitem': mapmapitem,

  'mapitems': mapitems,
  'mapmapitems': mapmapitems,

  'mapmethodcall': mapmethodcall,
  'mapmapmethodcall': mapmapmethodcall,

  'mapjoin': lambda seqs, sep: map(
    lambda seq: sep.join(map(str, seq)),
    seqs),

  'zip': lambda seq, *seqs: zip(*chain(seq, seqs)),
  'zipwith': zip,

  'tee': tee,

  'repeat': repeat,

  'chain': lambda seq, *seqs: chain(*chain(seq, seqs)),
  'chainwith': chain,

  'combine': combinations,
  'permutate': permutations,

  'cross': lambda seq, *seqs: product(*chain(seq, seqs)),
  'crosswith': product,

  'dict': dict,

  'dictremove': do_dictremove,
  'dictupdate': do_dictupdate,

  'partial': partial,

  'split': str.split,

  'prefix': do_prefix,
  'suffix': do_suffix,
  }



# This file was auto-generated by zetup
#
# https://bitbucket.org/userzimmermann/zetup.py


from __future__ import absolute_import

import sys
import os
from collections import OrderedDict

from zetup.version import Version
from zetup.dist import Distribution
from zetup.requires import Requirements
from zetup.extras import Extras
from zetup.package import Packages, Package
from zetup.notebook import Notebook



zfg = sys.modules[__name__]

LICENSE = 'LGPLv3'

TITLE = 'jinjatools'

PYTHON = ['2.7', '3.3', '3.4']

MODULES = []

KEYWORDS = ['jinja', 'jinja2', 'tools', 'template', 'macro', 'filter', 'test', 'moretools', 'scons', 'builder', 'build', 'django', 'web', 'html', 'python3']

PACKAGES = Packages([

Package('jinjatools',
  sources=[
    'env.py',
    '__init__.py',
    'ext.py',
    'filters.py'
    ],
  subpackages=[
    
Package('jinjatools.django',
  sources=[
    '__init__.py'
    ],
  subpackages=[
    
    ],
  )
,
    
Package('jinjatools.scons',
  sources=[
    '__init__.py'
    ],
  subpackages=[
    
    ],
  )

    ],
  )

  ], root=os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

EMAIL = 'zimmermann.code@gmail.com'

AUTHOR = 'Stefan Zimmermann'

REQUIRES = Requirements("""
six >= 1.9
path.py >= 7.5 #import path
moretools >= 0.1.7
Jinja2 #import jinja2
zetup >= 0.2.16

""", zfg=zfg)

URL = 'http://bitbucket.org/userzimmermann/python-jinjatools'

SETUP_HOOKS = []

VERSION = Version('0.1.6')

NAME = 'jinjatools'

DESCRIPTION = 'Various tools for Jinja2, including new filters and tests based on moretools, a JinjaLoader class for Django, and a simple JinjaBuilder class for SCons.'

SETUP_REQUIRES = None

EXTRAS = Extras([

], zfg=zfg)

DISTRIBUTION = Distribution('jinjatools', 'jinjatools', '0.1.6')

CLASSIFIERS = ['Development Status :: 3 - Alpha', 'Intended Audience :: Developers', 'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)', 'Operating System :: OS Independent', 'Programming Language :: Python', 'Programming Language :: Python :: 2', 'Programming Language :: Python :: 3', 'Topic :: Software Development', 'Topic :: Utilities', 'Programming Language :: Python', 'Programming Language :: Python :: 2.7', 'Programming Language :: Python :: 3.3', 'Programming Language :: Python :: 3.4']


# This file was auto-generated by zetup
#
# https://bitbucket.org/userzimmermann/zetup.py



from __future__ import print_function

import sys
import os
import re
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


SETUP_REQUIRES = [
  'zetup >= 0.2.16',
  
  ] + (os.path.exists('requirements.setup.txt')
       and [line for line in map(str.strip, open('requirements.setup.txt'))
            if line and not line.startswith('#')]
       or []) + [
  ]


try:
    from setuptools.dist import Distribution
    from pkg_resources import get_distribution, working_set, \
      DistributionNotFound, VersionConflict
except ImportError: # no setuptools
    pass
else:
    # make sure that setup requirements
    # are always correctly resolved and accessible by:
    # - pre-processing them one after another
    # - recursively resolving their runtime requirements
    # - moving any installed eggs to the front of sys.path
    # - updating pkg_resources.working_set accordingly

    installer = Distribution().fetch_build_egg

    def resolve(requirements, parent=None):
        for req in requirements:
            qualreq = parent and '%s->%s' % (req, parent) or req
            print("Resolving setup requirement %s:" % qualreq)
            try:
                dist = get_distribution(req)
                print(repr(dist))
            except (DistributionNotFound, VersionConflict):
                dist = installer(req)
                sys.path.insert(0, dist.location)
                working_set.entries.insert(0, dist.location)
                working_set.by_key[dist.key] = dist
            extras = re.match(r'[^#\[]*\[([^#\]]*)\]', req)
            if extras:
                extras = list(map(str.strip, extras.group(1).split(',')))
            resolve(map(str, dist.requires(extras=extras or ())), qualreq)

    resolve(SETUP_REQUIRES)


dist = setup(
  
  setup_requires=SETUP_REQUIRES,

  use_zetup=True,
  
  )


# python-jinjatools
#
# Various tools for Jinja2,
# including new filters and tests based on python-moretools,
# a JinjaLoader class for Django,
# and a simple JinjaBuilder class for SCons.
#
# Copyright (C) 2011-2015 Stefan Zimmermann <zimmermann.code@gmail.com>
#
# python-jinjatools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-jinjatools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with python-jinjatools.  If not, see <http://www.gnu.org/licenses/>.

from six import add_metaclass

from jinja2.ext import Extension
from jinja2.nodes import CallBlock


class TagExtensionMeta(type(Extension)):
    def __new__(mcs, clsname, bases, clsattrs):
        tagtree = {}
        for name in clsattrs:
            if name.startswith('tag_'):
                level = tagtree
                for tag in name.split('_')[1:]:
                    level = level.setdefault(tag, {})

        clsattrs['tags'] = set(tagtree)
        cls = type(Extension).__new__(mcs, clsname, bases, clsattrs)

        def parse_subtags(self, tag, parser):
            level = tagtree[tag]
            subtags = []
            while level:
                token = parser.stream.current
                if token.type != 'name':
                    break
                try:
                    level = level[token.value]
                except KeyError:
                    break

                subtags.append(token.value)
                parser.stream.skip()

            return subtags

        cls._parse_subtags = parse_subtags
        return cls


@add_metaclass(TagExtensionMeta)
class TagExtension(Extension):

    def parse(self, parser):
        tag = parser.stream.current.value
        parser.stream.next()

        subtags = self._parse_subtags(tag, parser)
        methodname = 'tag_' + tag
        if subtags:
            methodname += '_' + '_'.join(subtags)

        lineno = parser.stream.current.lineno
        content = parser.parse_statements(
          ['name:end' + tag], drop_needle=True)
        node = CallBlock(self.call_method(methodname), [], [], content)
        node.set_lineno(lineno)
        return node
